import 'dart:convert';
import 'dart:developer';
import 'dart:io';

import 'package:dio/dio.dart';
import 'package:sprintf/sprintf.dart';

class AuthInterceptor extends Interceptor {
  @override
  Future onRequest(
      RequestOptions options, RequestInterceptorHandler handler) async {
    super.onRequest(options, handler);
  }

  @override
  Future onResponse(
      Response response, ResponseInterceptorHandler handler) async {
    print("---->${response.data}");
    final RequestOptions request = response.requestOptions;
    return super.onResponse(response, handler);
  }
}

class LoggingInterceptor extends Interceptor {
  DateTime? _startTime;
  DateTime? _endTime;

  @override
  Future<void> onRequest(
      RequestOptions options, RequestInterceptorHandler handler) async {
    _startTime = DateTime.now();
    log('----------Start----------');
    if (options.queryParameters.isEmpty) {
      log('RequestUrl: ' + options.baseUrl + options.path);
    } else {
      log('RequestUrl: ' +
          options.baseUrl +
          options.path +
          '?' +
          Transformer.urlEncodeMap(options.queryParameters));
    }
    log('RequestMethod: ' + options.method);
    log('RequestHeaders:' + options.headers.toString());
    log('RequestContentType: ${options.contentType}');
    log('RequestData: ${options.data.toString()}');
    super.onRequest(options, handler);
  }

  @override
  Future onResponse(
      Response response, ResponseInterceptorHandler handler) async {
    _endTime = DateTime.now();
    final int duration = _endTime!.difference(_startTime!).inMilliseconds;
    // 输出结果
    log('----------End: $duration 毫秒----------');
    return super.onResponse(response, handler);
  }

  @override
  Future<void> onError(DioError err, ErrorInterceptorHandler handler) async {
    log('----------Error-----------');
    super.onError(err, handler);
  }
}

class AdapterInterceptor extends Interceptor {
  static const String _kMsg = 'msg';
  static const String _kSlash = '\'';
  static const String _kMessage = 'message';

  static const String _kDefaultText = '"无返回信息"';
  static const String _kNotFound = '未找到查询信息';

  static const String _kFailureFormat = '{"code":%d,"message":"%s"}';
  static const String _kSuccessFormat = '{"code":0,"data":%s,"message":""}';

  @override
  void onResponse(Response response, ResponseInterceptorHandler handler) {
    final Response r = adapterData(response);
    super.onResponse(r, handler);
  }

  @override
  void onError(DioError err, ErrorInterceptorHandler handler) {
    if (err.response != null) {
      adapterData(err.response!);
    }
    super.onError(err, handler);
  }

  Response<dynamic> adapterData(Response response) {
    String result;
    String content = response.data?.toString() ?? '';

    if (content.isEmpty) {
      // 一般为网络断开等异常
      result = content;
    } else {
      String msg;
      try {
        content = content.replaceAll(r'\', '');
        if (_kSlash == content.substring(0, 1)) {
          content = content.substring(1, content.length - 1);
        }
        final Map<String, dynamic> map =
            json.decode(content) as Map<String, dynamic>;
        if (map.containsKey(_kMessage)) {
          msg = map[_kMessage] as String;
        } else if (map.containsKey(_kMsg)) {
          msg = map[_kMsg] as String;
        } else {
          msg = '未知异常';
        }
        result = sprintf(_kFailureFormat, [response.statusCode, msg]);
      } catch (e) {
//            Log.d('异常信息：$e');
        // 解析异常直接按照返回原数据处理（一般为返回500,503 HTML页面代码）
        result = sprintf(_kFailureFormat,
            [response.statusCode, '服务器异常(${response.statusCode})']);
      }
    }
    response.data = result;
    return response;
  }
}
